
# search.py
# Andrew Davison, ad@coe.psu.ac.th, March 2025

'''
  Usage: python search.py "regex"  [-f] [-c]
    -f means print file and line no.
    -c means case of regex string is significant

  e.g. 
       python search.py "first"  # upper or lower case

       python search.py "Rob[a-z]+"      # upper or lower case
       python search.py "Rob[a-z]+"  -f  
       python search.py "Rob[a-z]+"  -c   # case matters

       python search.py "(Spit)|(Mess)"

       python search.py "Stories" -c

       python search.py "literary"
'''

import os, re, sys

INDEX_DIR = "Indices"



def searchFiles(directory, pattern, printFileInfo):
  regex = re.compile(pattern)
  linesPrinted = 0 
  for root, dirs, files in os.walk(directory):
    for file in files:
      filePath = os.path.join(root, file)
      fnm = os.path.splitext(file)[0]  # Strip off the extension
      # print(fnm)
      try:
        with open(filePath, 'r', encoding='utf-8') as f:
          lines = f.readlines()
          lineNum = 0
          while lineNum < len(lines):
            line = lines[lineNum]
            lineNum += 1
            
            # Skip lines that start with '#'
            if line.lstrip().startswith("#"):
              continue
            
            if regex.search(line):   # pattern matches
              if printFileInfo:
                print(f"{fnm:<10}:{lineNum:4d} > {line.rstrip()}")
              else:
                print(line.rstrip())
              linesPrinted += 1

              # If line ends with '$', keep printing following lines 
              # until one doesn't start with two spaces
              if line.rstrip().endswith("$"):
                while lineNum < len(lines):
                  nextLine = lines[lineNum]
                  lineNum += 1
                  if not nextLine.startswith("  "):
                    break
                  if printFileInfo:
                    print(f"{fnm:<10}:{lineNum:4d} > {nextLine.rstrip()}")
                  else:
                    print(nextLine.rstrip())
                  linesPrinted += 1

      
      except (IOError, UnicodeDecodeError) as e:
        print(f"Could not read file {filePath}: {e}")

  print("\nMatches:", linesPrinted)

# ---------------
if len(sys.argv) < 2:
  print("Usage: python search.py <pattern> [-f] [-i]")
  sys.exit(1)

pattern = sys.argv[1]
printFileInfo = "-f" in sys.argv  # Check if "-f" is present

useCase = "-c" in sys.argv 
if not useCase:
  pattern = "(?i)" + pattern

searchFiles(INDEX_DIR, pattern, printFileInfo)
